# Path & Name to Log- / Sensorfile(s)
Core="/sys/devices/platform/coretemp.0/"
i2c="/sys/bus/i2c/devices/0-002e/"

# IP for SNMP-Service:
SynoCom="syno"
Ip=127.0.0.1

# CPU-Sensoren:
declare -a cpu_name
declare -a cpu_value
cpu_count=0

# Lüfter-Sensoren
declare -a fan_name
declare -a fan_value
fan_count=0

# i2c Thermal-Sensoren
declare -a i2c_name
declare -a i2c_value
i2c_count=0

# PlotData Array
declare -A plotData

timerCount=0

# ----------------------------------------------------------
# --- Moving Avarage Array initialisieren
declare -A arr
InitArray 10 10

initLog()
{
    local avrDate=$(date '+%Y.%m.%d')
    local avrTime=$(date '+%H:%M:%S')

    escLog           "╠═ AvrLogger ══════════════════════════════════════════════════════╣"
    escLog "$(printf "║ AvrVer. =  %8.8s  SvgVer. =  %8.8s  Date    =  %10.10s  ║" "$AvrVersion" "$SvgVersion" "$avrDate")"
    escLog "$(printf "║ AvrTick =    %3.3sSec  SvgTick =    %3.3sSec  Time    =    %8.8s  ║" "$AvrTick" "$SvgTick" "$avrTime")"
    escLog "$(printf "║ AvrLog  =       %3.3s  AvrMode =    %6.6s  SvgLog  =         %3.3s  ║" "$AvrLogStatus" "$AvrLogMode" "$SvgStatus")"
    escLog           "╠═ Sensors ═════════════════════ Environment ══════════════════════╣"
    escLog "$(printf "║                FAN Sens. = %2d  ENV  = %3.3s   IP = %15.15s ║" "$fan_count" "$AvrEnvStatus" "$AvrEnvIp")"
    escLog "$(printf "║ CPU Sen. = %2d  i2c Sens. = %2d  Name = %26.26s ║" "$cpu_count" "$i2c_count" "$AvrEnvName")"
    escLog           "╠═ Syno ═════════════════════ CPU ═════════════════════════════════╣"
    escLog "$(printf "║ Model    = %15.15s  Manufacturer = %17.17s     ║" "${SynoModel[0]}" "${SynoCPU[0]}")"
    escLog "$(printf "║ Product  = %15.15s  Collection   = %17.17s     ║" "${SynoModel[1]}" "${SynoCPU[1]}")"
    escLog "$(printf "║ Platform =   %13.13s  Name         = %17.17s     ║" "${SynoModel[2]}" "${SynoCPU[2]}")"
    escLog "$(printf "║ Hostname = %15.15s  Clock        =           %1.2fGHz     ║" "${SynoModel[3]}" "${SynoCPU[5]}")"
    escLog "$(printf "║ Uptime   =   %13.13s  Cores   =  %2d   Threads   =  %2d      ║" "${SynoModel[4]}" "${SynoCPU[3]}" "${SynoCPU[4]}")"
    escLog           "╠═ Disk's ═════════════════════════════════════════════════════════╝"
}


avrExecution=0

# ----------------------------------------------------------
# --- Check if an instance of AvrLogger is already running
# ----------------------------------------------------------
avrCheckPid()
{
    local chkPID
    local listPID
    local PID
    # --- Check AvrLogger PID
    if [ -e "$workingDir""avr.pid" ]; then
        chkPID=`cat "$workingDir""avr.pid"`
       	listPID=`ps -e | grep "avr.sh" | grep -v grep | awk '{print $1;}'`
       	for PID in $listPID; do
           	if [ $PID -eq $chkPID ]
           	then
               	avrExecution=1
    			break
           	fi
       	done
    fi
}

# ----------------------------------------------------------
# --- Main-Initialization
# ----------------------------------------------------------
avrInit()
{

    if [ $avrExecution -eq 0 ]; then
	    # --- Store PID
	    AvrPID=$$
        echo "$AvrPID" > "$workingDir""avr.pid"
        # Read-Only
	    chmod 644 "$workingDir""avr.pid"
    fi

    # --- Path to Avr-Config
    AvrConf="$workingDir""config/avr.conf"
    # --- Path to SVG-Config
    SvgConf="$workingDir""config/svg.conf"

    if [ $avrTest -eq 0 ]
    then
        AvrLog="$workingDir""data/avr.csv"
        AvrDat="$workingDir""tmp/avr.dat"
        SvgFile="$workingDir""data/avr.svg"
    else
        AvrLog="$workingDir""data/test.csv"
        AvrDat="$workingDir""tmp/test.dat"
        SvgFile="$workingDir""data/test.svg"
    fi

    AvrService=$(GetValue "$AvrConf" "AvrService")
    if [ $avrStatus -eq 1 ]; then
        src="$workingDir""rc.d/avr_start.sh"
        dst="/usr/local/etc/rc.d/avr_start.sh"

        if [ "$AvrService" == "Yes" ]; then
            cp "$src" "$dst"
            PutValue "$dst" "workingDir" "$workingDir"
            chmod 755 "$dst"
        fi

        if [ "$AvrService" == "No" ]; then
            if [ -e "$dst" ]; then rm "$dst"; fi
        fi
    fi

    # --- Path to Synology cpuinfo
    SynoCpuInfo="/proc/cpuinfo"
    # --- Path to Synology synoinfo.conf
	SynoInfo="/etc/synoinfo.conf"

    # ----------------------------------------------------------
	# --- Get some CPU Infos
    # ---> SynoCpu[0] = Manufacturer
    # ---> SynoCpu[1] = Product Collection
    # ---> SynoCpu[2] = Processor Number
    # ---> SynoCpu[3] = # of Cores
    # ---> SynoCpu[4] = # of Threads
    # ---> SynoCpu[5] = Base Frequency GHz
    read -r -a SynoCPU <<< $(cat /proc/syno_cpu_arch | sed 's/[ ]//g' | sed 's/[,]/ /g')
    SynoCPU[4]=$(cat "/proc/cpuinfo" | grep -c '^processor')
    SynoCPU[5]=$(cat "/proc/cpuinfo" | grep -m 1 "cpu MHz" | awk '{print $4/1000}')

    # ----------------------------------------------------------
	# --- Get some Syno Infos
	# ---> SynoModel[0] = Synology Product
	# ---> SynoModel[1] = Product Type ("RackStation" / "DiskStation")
	# ---> SynoModel[2] = Platform
	# ---> SynoModel[3] = Hostname
	# ---> SynoModel[4] = Uptime Sec.
	SynoModel[0]=$(GetValue "$SynoInfo" "upnpmodelname")  # Synology Product "RS3617xs"
    SynoModel[1]=$(GetValue "$SynoInfo" "product")        # RackStation / DiskStation
    SynoModel[2]=$(cat /proc/syno_platform)
    SynoModel[3]=$(cat /etc/hostname)
    SynoModel[4]=$(cat /proc/uptime | awk '{print $1}' | sed -r 's/([^\.]*)\..*/\1/')

    uDay=$[SynoModel[4]/86400]
    SynoModel[4]=$[SynoModel[4]-(uDay*86400)]
    uHour=$[SynoModel[4]/3600]
    SynoModel[4]=$[SynoModel[4]-(uHour*3600)]
    uMin=$[SynoModel[4]/60]
    SynoModel[4]=$[SynoModel[4]-(uMin*60)]
    uSec=${SynoModel[4]}
    SynoModel[4]="$(printf "%4d:%02d:%02d:%02d" "$uDay" "$uHour" "$uMin" "$uSec")"

    SynoSnmp=$(GetValue "$SynoInfo" "runsnmpd")          # SNMP-Service "yes" / "no"

    # ----------------------------------------------------------
	# --- Get values from Avr-Config
    # ----------------------------------------------------------
    # --- Some Switches "Yes/No"
    # --- Flag: '(with BOM)'
    AvrLogBOM=$(GetValue "$AvrConf" "AvrLogBOM")
    if [ "$AvrLogBOM" == "Yes" ]
        then AvrLogBOM=1
        else AvrLogBOM=0
    fi

    # --- Flag: AvrLogOn
	AvrLogOn=$(GetValue "$AvrConf" "AvrLogOn")
    if [[ "$AvrLogOn" == "Yes" || $avrTest -eq 1 ]]
    then
        AvrLogOn=1
        AvrLogStatus="ON"
    else
        AvrLogOn=0
        AvrLogStatus="OFF"
    fi

    if [ $avrTest -eq 1 ]
    then
        AvrLogMode="Free"
    else
        AvrLogMode=$(GetValue "$AvrConf" "AvrLogMode")
        if [[ "$AvrLogMode" != "Daily" && "$AvrLogMode" != "Moving" && "$AvrLogMode" != "Free" ]]; then
            AvrLogOn=0
            AvrLogStatus="OFF"
            AvrLogMode="ERROR"
        fi
    fi

    # --- Flag: Enable CSV-Archive
	AvrArchive=$(GetValue "$AvrConf" "AvrArchive")
    if [ "$AvrArchive" == "Yes" ]
        then AvrArchive=1
        else AvrArchive=0
    fi

    escLog "╔═══════════════════════════════════════════════════ Avr Start Up ═╗"
    escLog "$(printf "║ AVR: AvrLogger was started in %8.8s-Mode                      ║" "$avrStartMode")"
    escLog "$(printf "║ AVR: avrDebug = %1d  avrInfo = %1d  avrTest = %1d                      ║" "$avrDebug" "$avrInfo" "$avrTest")"

####################################################################

    # --- Flag: Enable SVG
	SvgOn=$(GetValue "$AvrConf" "SvgOn")
    if [[ "$SvgOn" == "Yes" || $avrTest -eq 1 ]]
    then
        SvgOn=1
        SvgStatus="ON"
    else
        SvgOn=0
        SvgStatus="OFF"
    fi

    # --- Flag: Enable SVG-Archive
	SvgArchive=$(GetValue "$AvrConf" "SvgArchive")
    if [ "$SvgArchive" == "Yes" ]
        then SvgArchive=1
        else SvgArchive=0
    fi

    # --- Flag: Enable SMART-Use
	UseSmart=$(GetValue "$AvrConf" "AvrSmart")
    if [ "$UseSmart" == "Yes" ]
        then UseSmart=1
        else UseSmart=0
    fi

    # --- Some Values: Community-String for SNMP-Function
	Com=$(GetValue "$AvrConf" "AvrCommunity")

    # --- FAN Speed
    Rpm=$(GetValue "$AvrConf" "FanSpeed")

    # --- Get AvrTick / SvgTick Time
    if [[ $avrExecution -eq 0 && $avrTest -eq 0 ]]; then
   	    AvrTick=$(GetValue "$AvrConf" "AvrTick")
   	fi
    SvgTick=$(GetValue "$AvrConf" "SvgTick")

    # --- Separator for CSV-Column's
	ColSign=$(GetValue "$AvrConf" "AvrColSign")
	# --- Decimal Point for numerical CSV-Values
	DecSign=$(GetValue "$AvrConf" "AvrDecSign")

	# --- Environmental sensor Name, Ip & OID
	AvrEnvName=$(GetValue "$AvrConf" "AvrEnvName")
    if [ ${#AvrEnvName} -eq 0 ]
    then
        AvrEnv=0
        AvrEnvIp="000.000.000.000"
	    AvrEnvOID="not available"
	    AvrEnvStatus="No"
    else
        AvrEnv=1
    	AvrEnvIp=$(GetValue "$AvrConf" "AvrEnvIp")
	    AvrEnvOID=$(GetValue "$AvrConf" "AvrEnvOID")
	    AvrEnvStatus="Yes"
    fi

    addNewLines=0
}

# ----------------------------------------------------------
# Print a list of importand Disk-Informations.
# ----------------------------------------------------------
ListDisk()
{
    # ---------------------------------------------------
    # --- Collect Disk's Data
    # ---------------------------------------------------
    TimerStart
    dsk_te=($(snmpwalk -v 2c  -c $SynoCom -O qv $Ip 1.3.6.1.4.1.6574.2.1.1.6))
    SynoDisks=${#dsk_te[@]}

    escPrint 14 6 "$(printf "%2.2s" $SynoDisks)"

    escEcho "ID      | Type     | Model                    | °C   | Loop      "
    escEcho "--------+----------+--------------------------+------------------"

    for(( m=0; m<$SynoDisks; m++)) do
        TimerStart
        # --- "disk id"
        id=$(snmpwalk -v 2c  -c $SynoCom -O qv $Ip 1.3.6.1.4.1.6574.2.1.1.2.$m)
        id=$(echo $id | sed 's/^[^"]*"//' | sed  's/".*//' | grep -o '[0-9]*')
        # --- "disk model's"
        mo=$(snmpwalk -v 2c  -c $SynoCom -O qv $Ip 1.3.6.1.4.1.6574.2.1.1.3.$m)
        mo=$(echo $mo | sed 's/^[^"]*"//' | sed  's/".*//')
        # --- "disk type's"
        ty=$(snmpwalk -v 2c  -c $SynoCom -O qv $Ip 1.3.6.1.4.1.6574.2.1.1.4.$m)
        ty=$(echo $ty | sed 's/^[^"]*"//' | sed  's/".*//')
        # --- "disk temperature's"
        te=$(snmpwalk -v 2c  -c $SynoCom -O qv $Ip 1.3.6.1.4.1.6574.2.1.1.6.$m)

        TimerStop
        escEcho "$(printf "Disk%3d | %-8.8s | %-24.24s | %2d°C |" $id "$ty" "$mo" $te)" "$timerLap"
    done

    TimerStop
    escEcho "----------------------------------------------------------------"
    escEcho 'SYS: "'$SynoDisks'" Disk Infos collected' "$timerLap"
}

# ----------------------------------------------------------
# --- Moving Avarage Array initialisieren
InitArray()
{
	arr_row=$1	# 10
	arr_col=$2	# 10
	arr_cnt=0

	for ((i=0;i<=arr_row;i++)) do
		for ((j=0;j<=arr_col;j++)) do
	    	arr[$i,$j]=0
		done
	done
}

# ----------------------------------------------------------
# --- Writes Headline to AVR-Logfile
# --- $1 = Headline
WriteHeadline()
{
    # --- Echo HeadLine
    if [[ $AvrLogOn -eq 1 || $avrInfo -eq 1 ]]; then
        if [ ! -e "$AvrLog" ]; then
            if [ $AvrLogBOM -eq 1 ]
                # --- Mark as '(with BOM)'
                then printf '\xEF\xBB\xBF' > "$AvrLog"
                echo $1 >> "$AvrLog"
            fi
            # --- to CSV-File
            echo $1 > "$AvrLog"
        fi
    fi
}

